/*
 *
 * my_qu_ix_w.h
 *
 * Code generation for function 'my_qu_ix_w'
 *
 */

#ifndef MY_QU_IX_W_H
#define MY_QU_IX_W_H

/* Include files */
#include <stddef.h>
#include <stdlib.h>
#include "rtwtypes.h"
#include "omp.h"
#include "fetch_thresholds_types.h"

/* Function Declarations */
extern double my_qu_ix_w(const emxArray_real_T *invec, const emxArray_real_T
  *yinwec);

#endif

/* End of code generation (my_qu_ix_w.h) */
