/*
 *
 * fetch_thresholds_data.c
 *
 * Code generation for function 'fetch_thresholds_data'
 *
 */

/* Include files */
#include "fetch_thresholds_data.h"
#include "fetch_thresholds.h"
#include "rt_nonfinite.h"

/* Variable Definitions */
omp_nest_lock_t emlrtNestLockGlobal;
boolean_T isInitialized_fetch_thresholds = false;

/* End of code generation (fetch_thresholds_data.c) */
